<?php
/*
Plugin Name: Contact Pop-up
Plugin URI: http://www.20script.ir
Description: A handy small Contact Form that's on every page of your site.
Version: 1.6
Author: Jordie Bodlay
Author URI: http://www.20script.ir
License: Commercial License
*/


global $cpu_options, $cpu_db_version;

$cpu_db_version = '1.6';

// Load the language files
load_plugin_textdomain('contact-popup', false, basename( dirname( __FILE__ ) ) . '/lang' );

include(dirname(__FILE__) . '/settings.php');

register_activation_hook(__FILE__, 'install_ultimate_contact_human');

add_action('wp_enqueue_scripts', 'cpu_scripts');


// if user signed in
add_action( 'wp_ajax_contact_popup_send', 'cpu_send_message' );
// if user not signed in
add_action( 'wp_ajax_nopriv_contact_popup_send', 'cpu_send_message' );

add_action( 'admin_menu', 'cpu_menu' );

function cpu_menu() {
	add_options_page( 'Contact Popup Options', 'Contact Popup ', 'manage_options', 'contact-popup-options', 'contact_popup_options_page' );
}



/**
 * Settings page in the WP Admin
 */
function contact_popup_options_page() {
	if ( !current_user_can( 'manage_options' ) )  {
		wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
	}


	wp_enqueue_script( 'jquery-ui', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/jquery-ui.min.js' );
	wp_enqueue_script( 'ultimate_contact_page_colorpicker', plugins_url('/js/jquery.ui.colorPicker.min.js', __FILE__) );

	wp_register_style('jquery-ui-css', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/themes/ui-lightness/jquery-ui.css' );
	wp_enqueue_style( 'jquery-ui-css' );
	wp_register_style('colorpicker-ui-css', plugins_url('/css/jquery.ui.colorPicker.css', __FILE__));
	wp_enqueue_style( 'colorpicker-ui-css' );

	include(dirname(__FILE__) . '/options-page.php');

}

add_action( 'plugins_loaded',	'cpu_update_db_check');

function cpu_update_db_check() {
	global $cpu_db_version;

	if (get_option('cpu_db_version') != $cpu_db_version) {
		install_contact_popup_human();
	}
}


add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'cpu_plugin_action_links', 10, 1);
function cpu_plugin_action_links($links) {
	$settings_page = add_query_arg(array('page' => 'contact-popup-options'), admin_url('options-general.php'));
	$settings_link = '<a href="'.esc_url($settings_page).'">'.__('Settings', 'contact-popup' ).'</a>';
	array_unshift($links, $settings_link);
	return $links;
}

function install_contact_popup_human() {
	global $wpdb;
	global $cpu_db_version;
	$installed_ver = get_option( "cpu_db_version" );

	if( $installed_ver != $cpu_db_version ) {
		$table_name = $wpdb->prefix . "cpu_humantest";

		if((float)$installed_ver < 1.6 && (float)$installed_ver > 0.9) {
			$res = $wpdb->get_results("SHOW INDEX FROM ".$table_name." WHERE `Key_name` LIKE 'testkey%'");

			foreach($res as $row) {
				$wpdb->query("DROP INDEX `{$row->Key_name}` ON `{$row->Table}`");
			}
		}


		$sql = "CREATE TABLE ". $table_name ." (
			testkey VARCHAR(255) DEFAULT '' NOT NULL,
			result tinytext NOT NULL,
			createdtime VARCHAR(255) DEFAULT '' NOT NULL,
			PRIMARY KEY (testkey)
		);";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);

		if(empty($installed_ver)) {
			add_option("cpu_db_version", $cpu_db_version);
		} else { 
			update_option( "cpu_db_version", $cpu_db_version );
		}
	}
}

add_action('wp_footer', 'cpu_main_html');

function cpu_main_html() {
	global $wpdb;

	$options = cpu_get_options();
	if($options['cpu_enable_test'] == 'enable') {
		// generate a math question for the human test
		$number1 = rand(1,10);
		$number2 = rand(1,10);
		$result = $number1 + $number2;
		$test_key = cpu_generateRandomString();

		$insert_query = "INSERT INTO ".$wpdb->prefix ."cpu_humantest ( testkey, result, createdtime) VALUES (%s, %s, %s )";
		$wpdb->query( $wpdb->prepare( $insert_query , $test_key, $result, time()) );

		$del_query = "DELETE FROM ".$wpdb->prefix ."cpu_humantest WHERE createdtime < " . (time() - (86400*3));
		$wpdb->query( $del_query );
	}

	include(dirname(__FILE__) . '/contact-form.php');
}


function cpu_scripts() {
	wp_enqueue_script(
		'contact_popup_send',
		plugins_url('/js/contact-popup.js', __FILE__),
		array('jquery')
	);
	wp_localize_script( 'contact_popup_send', 'ContactPopup', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );

	wp_register_style( 'cpu_styles', plugins_url('/css/contact-popup.css', __FILE__) );
	wp_enqueue_style( 'cpu_styles' );
}

function cpu_generateRandomString($length = 10) {
	return substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"), 0, $length);
}

function cpu_send_message() {
	global $wpdb;
	header( "Content-Type: application/json" );
	$return = array();

	$options = cpu_get_options();
	if(empty($_POST['name'])) {
		echo json_encode(array('success' => false, 'message' => __("Message not sent: Please enter your name.", 'contact-popup' )));
		die();
	}

	if(empty($_POST['email'])) {
		echo json_encode(array('success' => false, 'message' => __("Message not sent: Please enter your email.", 'contact-popup' )));
		die();
	}

	if(empty($_POST['message'])) {
		echo json_encode(array('success' => false, 'message' => __("Message not sent: Please enter your message.", 'contact-popup' )));
		die();
	}



	if($options['cpu_enable_test'] == 'enable') {
		$my_test = $wpdb->get_row($wpdb->prepare( "SELECT * FROM ". $wpdb->prefix . "cpu_humantest WHERE testkey = %s", $_POST['test_key']) );
		if(!is_object($my_test) || (int)$my_test->result !== (int)$_POST['human_test'] || empty($_POST['test_key'])) {
			echo json_encode( array( 'success' => false, 'message' => __('The answer to the human test was not correct.','contact-popup'), 'reason' => 'badtest' ) );
			die();
		}
	}


	if(isset($_POST['subscribe']) && $_POST['subscribe'] == 'yes') {
		require_once(dirname(__FILE__) . '/lib/MCAPI.class.php');
		$api = new MCAPI(@$options['cpu_mc_api']);

		if($api->listSubscribe(@$options['cpu_mc_list'], $_POST['email'], '') === true) {
			// It worked!
			$mailchimp = true;
		}else{
			// An error ocurred, return error message
			$mailchimp = false;
		}
	}

	// get the submitted parameters
	$headers = array();
	$headers[] = 'From: ' .$_POST['name'] . ' <' . str_replace(array("\r", "\n", "\n", "\t", ",", ";"), '', $_POST['email']). ">\r\n";
	$result = wp_mail(@$options['cpu_email'], get_bloginfo('name') . __( " Contact Form Submission", 'contact-popup' ), "The following is a message sent by ". $_POST['name'] . ' <' . str_replace(array("\r", "\n", "\n", "\t", ",", ";"), '', $_POST['email']). "> from your ". get_bloginfo('name')." contact form:\n----------------------------\n" .$_POST['message'], $headers);

	if($result) {
		echo json_encode(array('success' => true));
		die();
	}

	echo json_encode(array('success' => false, 'message' => __("Message not sent. An unknown error occurred.", 'contact-popup' ), 'result' => $result));
		die();

}